use crate::model::{
    boat::{Boat, BoatToAdd, BoatToUpdate},
    location::Location,
    user::{AdminUser, User},
};
use rocket::{
    form::Form,
    get, post,
    request::FlashMessage,
    response::{Flash, Redirect},
    routes, Route, State,
};
use rocket_dyn_templates::{tera::Context, Template};
use sqlx::SqlitePool;

#[get("/boat")]
async fn index(
    db: &State<SqlitePool>,
    admin: AdminUser,
    flash: Option<FlashMessage<'_>>,
) -> Template {
    let boats = Boat::all(db).await;
    let locations = Location::all(db).await;
    let users = User::all(db).await;

    let mut context = Context::new();
    if let Some(msg) = flash {
        context.insert("flash", &msg.into_inner());
    }
    context.insert("boats", &boats);
    context.insert("locations", &locations);
    context.insert("users", &users);
    context.insert("loggedin_user", &admin.user);

    Template::render("admin/boat/index", context.into_json())
}

#[get("/boat/<boat>/delete")]
async fn delete(db: &State<SqlitePool>, _admin: AdminUser, boat: i32) -> Flash<Redirect> {
    let boat = Boat::find_by_id(db, boat).await;
    match boat {
        Some(boat) => {
            boat.delete(db).await;
            Flash::success(
                Redirect::to("/admin/boat"),
                format!("Sucessfully deleted boat {}", boat.name),
            )
        }
        None => Flash::error(Redirect::to("/admin/boat"), "Boat does not exist"),
    }
}

#[post("/boat", data = "<data>")]
async fn update(
    db: &State<SqlitePool>,
    data: Form<BoatToUpdate<'_>>,
    _admin: AdminUser,
) -> Flash<Redirect> {
    let boat = Boat::find_by_id(db, data.id).await;
    let Some(boat) = boat else {
            return Flash::error(
                Redirect::to("/admin/boat"),
                "Boat does not exist!",
            )
    };

    if !boat.update(db, data.into_inner()).await {
        return Flash::error(Redirect::to("/admin/boat"), "Boat could not be updated!");
    }

    Flash::success(Redirect::to("/admin/boat"), "Successfully updated boat")
}

#[post("/boat/new", data = "<data>")]
async fn create(
    db: &State<SqlitePool>,
    data: Form<BoatToAdd<'_>>,
    _admin: AdminUser,
) -> Flash<Redirect> {
    if Boat::create(db, data.into_inner()).await {
        Flash::success(Redirect::to("/admin/boat"), "Successfully created boat")
    } else {
        Flash::error(
            Redirect::to("/admin/boat"),
            "Error while creating the boat in DB",
        )
    }
}

pub fn routes() -> Vec<Route> {
    routes![index, create, delete, update]
}
